/*
 * Flash.c
 *
 *  Created on: 2024516
 *      Author: 34509
 */
#include <flash.h>

/* Global Variable */
u8 SPI_FLASH_BUF[4096];

/*******************************************************************************
* Function Name  : SPI_Flash_Init
* Description    : Configuring the SPI for operation flash.
* Input          : None
* Return         : None
*******************************************************************************/
void SPI_Flash_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    SPI_InitTypeDef  SPI_InitStructure;

    RCC_PB2PeriphClockCmd(RCC_APB2Periph_GPIOX, ENABLE );
    RCC_PB1PeriphClockCmd(RCC_APB1Periph_SPIX, ENABLE );

    GPIO_InitStructure.GPIO_Pin = CS_PIN;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;      //
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(CS_GPIO, &GPIO_InitStructure);
    GPIO_SetBits(CS_GPIO, CS_PIN);                      //ѡFLASH

    GPIO_InitStructure.GPIO_Pin = CLK_PIN;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;       //
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(CLK_GPIO, &GPIO_InitStructure );

    GPIO_InitStructure.GPIO_Pin = DO_PIN;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING; //
    GPIO_Init(DO_GPIO, &GPIO_InitStructure );

    GPIO_InitStructure.GPIO_Pin = DI_PIN;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;       //
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(DI_GPIO, &GPIO_InitStructure );

    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex; //SPIͨѶΪ˫ȫ˫ʽ
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;      //SPIΪģʽ
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;  //SPIͨѶ֡СΪ8λ
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_High;        //SPIʱӼΪߵƽ
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge;       //SPIʱλΪSCKżزɼ
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;          //NSSţƬѡţʹģʽΪģʽ
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4; //òʷƵΪ4Ƶ
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB; //ݴΪλǰ
    SPI_InitStructure.SPI_CRCPolynomial = 7;           //SPICRCУжʽֵ
    SPI_Init(SPIX, &SPI_InitStructure);                //ʼSPI

    SPI_Cmd(SPIX, ENABLE);                             //ʹSPI2
}

/*******************************************************************************
* Function Name  : SPIX_ReadWriteByte
* Description    : SPIX read or write one byte.
* Input          : TxData: write one byte data.
* Return         : Read one byte data.
*******************************************************************************/
u8 SPI_ReadWriteByte(u8 TxData)
{
    u8 i=0;
    while (SPI_I2S_GetFlagStatus(SPIX, SPI_I2S_FLAG_TXE) == RESET)  //ȴͻΪգTXE¼
    {
        i++;
        if(i>200)return 0;
    }
    SPI_I2S_SendData(SPIX, TxData);    //дݼĴҪдд뷢ͻͨSPI1һ
    i=0;
    while (SPI_I2S_GetFlagStatus(SPIX, SPI_I2S_FLAG_RXNE) == RESET) //ȴջǿգRXNE¼
    {
        i++;
        if(i>200)return 0;
    }
    return SPI_I2S_ReceiveData(SPIX);  //ȡݼĴȡջݣSPIյ
}

/*******************************************************************************
* Function Name  : SPI_Flash_ReadSR
* Description    : Read W25Qxx status register.
*       BIT7  6   5   4   3   2   1   0
*       SPR   RV  TB  BP2 BP1 BP0 WEL BUSY
* Input          : None
* Return         : byte: status register value.
*******************************************************************************/
u8 SPI_Flash_ReadSR(void)
{
    u8 byte=0;

    GPIO_WriteBit(CS_GPIO, CS_PIN, 0);    //PA2=0,ʹƬѡź
    SPI_ReadWriteByte(W25X_ReadStatusReg); //Ͷ״̬Ĵ
    byte=SPI_ReadWriteByte(0Xff);          //ȡһֽ
    GPIO_WriteBit(CS_GPIO, CS_PIN, 1);    //ȡƬѡź

    return byte;
}

/*******************************************************************************
* Function Name  : SPI_FLASH_Write_SR
* Description    : Write W25Qxx status register.
* Input          : sr:status register value.
* Return         : None
*******************************************************************************/
void SPI_FLASH_Write_SR(u8 sr)
{
    GPIO_WriteBit(CS_GPIO, CS_PIN, 0);
    SPI_ReadWriteByte(W25X_WriteStatusReg); //д״̬Ĵ
    SPI_ReadWriteByte(sr);                  //дһֽ
    GPIO_WriteBit(CS_GPIO, CS_PIN, 1);
}

/*******************************************************************************
* Function Name  : SPI_Flash_Wait_Busy
* Description    : Wait flash free.
* Input          : None
* Return         : None
*******************************************************************************/
void SPI_Flash_Wait_Busy(void)
{
    while((SPI_Flash_ReadSR()&0x01)==0x01); //ȴFLASH
}

/*******************************************************************************
* Function Name  : SPI_FLASH_Write_Enable
* Description    : Enable flash write.
* Input          : None
* Return         : None
*******************************************************************************/
void SPI_FLASH_Write_Enable(void)
{
  GPIO_WriteBit(CS_GPIO, CS_PIN, 0);
  SPI_ReadWriteByte(W25X_WriteEnable); //дʹ
  GPIO_WriteBit(CS_GPIO, CS_PIN, 1);
}

/*******************************************************************************
* Function Name  : SPI_FLASH_Write_Disable
* Description    : Disable flash write.
* Input          : None
* Return         : None
*******************************************************************************/
void SPI_FLASH_Write_Disable(void)
{
  GPIO_WriteBit(CS_GPIO, CS_PIN, 0);
  SPI_ReadWriteByte(W25X_WriteDisable); //дָֹ
  GPIO_WriteBit(CS_GPIO, CS_PIN, 1);
}

/*******************************************************************************
* Function Name  : SPI_Flash_ReadID
* Description    : Read flash ID.
* Input          : None
* Return         : Temp: FLASH ID.
*******************************************************************************/
u16 SPI_Flash_ReadID(void)
{
    u16 Temp = 0;

    GPIO_WriteBit(CS_GPIO, CS_PIN, 0);
    SPI_ReadWriteByte(W25X_ManufactDeviceID); //ͶȡID
    SPI_ReadWriteByte(0x00);
    SPI_ReadWriteByte(0x00);
    SPI_ReadWriteByte(0x00);
    Temp|=SPI_ReadWriteByte(0xFF)<<8;
    Temp|=SPI_ReadWriteByte(0xFF);
    GPIO_WriteBit(CS_GPIO, CS_PIN, 1);

    return Temp;
}

/*******************************************************************************
* Function Name  : SPI_Flash_Erase_Sector
* Description    : Erase one sector(4Kbyte).
* Input          : Dst_Addr:  0  2047
* Return         : None
*******************************************************************************/
void SPI_Flash_Erase_Sector(u32 Dst_Addr)
{
    Dst_Addr*=4096;
    SPI_FLASH_Write_Enable();
    SPI_Flash_Wait_Busy();
    GPIO_WriteBit(CS_GPIO, CS_PIN, 0);
    SPI_ReadWriteByte(W25X_SectorErase);     //ָ
    SPI_ReadWriteByte((u8)((Dst_Addr)>>16)); //24bitַ
    SPI_ReadWriteByte((u8)((Dst_Addr)>>8));
    SPI_ReadWriteByte((u8)Dst_Addr);
    GPIO_WriteBit(CS_GPIO, CS_PIN, 1);
    SPI_Flash_Wait_Busy(); //ȴ
}

/*******************************************************************************
* Function Name  : SPI_Flash_Read
* Description    : Read data from flash.
* Input          : pBuffer:
*                  ReadAddr:Initial address(24bit).
*                  size: Data length.
* Return         : None
*******************************************************************************/
void SPI_Flash_Read(u8* pBuffer,u32 ReadAddr,u16 size)
{
  u16 i;

  GPIO_WriteBit(CS_GPIO, CS_PIN, 0);
  SPI_ReadWriteByte(W25X_ReadData);        //Ͷȡ
  SPI_ReadWriteByte((u8)((ReadAddr)>>16)); //24bitַ
  SPI_ReadWriteByte((u8)((ReadAddr)>>8));
  SPI_ReadWriteByte((u8)ReadAddr);

  for(i=0;i<size;i++)
  {
     pBuffer[i] = SPI_ReadWriteByte(0XFF); //ѭ
  }

  GPIO_WriteBit(CS_GPIO, CS_PIN, 1);
}

/*******************************************************************************
* Function Name  : SPI_Flash_Write_Page
* Description    : Write data by one page.
* Input          : pBuffer:
*                  WriteAddr:Initial address(24bit).
*                  size:Data length.
* Return         : None
*******************************************************************************/
void SPI_Flash_Write_Page(u8* pBuffer,u32 WriteAddr,u16 size)
{
  u16 i;

  SPI_FLASH_Write_Enable();
  GPIO_WriteBit(CS_GPIO, CS_PIN, 0);
  SPI_ReadWriteByte(W25X_PageProgram);      //дҳ
  SPI_ReadWriteByte((u8)((WriteAddr)>>16)); //24bitַ
  SPI_ReadWriteByte((u8)((WriteAddr)>>8));
  SPI_ReadWriteByte((u8)WriteAddr);

  for(i=0;i<size;i++)
    {
      SPI_ReadWriteByte(pBuffer[i]); //ѭд
    }

  GPIO_WriteBit(CS_GPIO, CS_PIN, 1);
  SPI_Flash_Wait_Busy(); //ȴд
}

/*******************************************************************************
* Function Name  : SPI_Flash_Write_NoCheck
* Description    : Write data to flash.(need Erase)
*                  All data in address rang is 0xFF.
* Input          : pBuffer:
*                  WriteAddr: Initial address(24bit).
*                  size: Data length.
* Return         : None
*******************************************************************************/
void SPI_Flash_Write_NoCheck(u8* pBuffer,u32 WriteAddr,u16 size)
{
    u16 pageremain;

    pageremain=256-WriteAddr%256; //ҳʣֽ

    if(size<=pageremain)pageremain=size; //256ֽ

    while(1)
    {
        SPI_Flash_Write_Page(pBuffer,WriteAddr,pageremain);

        if(size==pageremain)
        {
            break; //д
        }
        else
        {
            pBuffer+=pageremain;
            WriteAddr+=pageremain;
            size-=pageremain; //ȥѾд˵ֽ

            if(size>256)pageremain=256; //һοд256ֽ
            else pageremain=size; //256ֽ
        }
    }
}

/*******************************************************************************
* Function Name  : SPI_Flash_Write
* Description    : Write data to flash.(no need Erase)
* Input          : pBuffer:
*                  WriteAddr: Initial address(24bit).
*                  size: Data length.
* Return         : None
*******************************************************************************/
void SPI_Flash_Write(u8* pBuffer,u32 WriteAddr,u16 size)
{
    u32 secpos;
    u16 secoff;
    u16 secremain;
    u16 i;

    secpos=WriteAddr/4096; //ַ
    secoff=WriteAddr%4096; //ڵı
    secremain=4096-secoff; //ʣռС

    if(size<=secremain)secremain=size; //4096ֽ

    while(1)
    {
        SPI_Flash_Read(SPI_FLASH_BUF,secpos*4096,4096); //

        for(i=0;i<secremain;i++) //У
        {
            if(SPI_FLASH_BUF[secoff+i]!=0XFF)break; //Ҫ
        }

        if(i<secremain) //Ҫ
        {
            SPI_Flash_Erase_Sector(secpos); //

            for(i=0;i<secremain;i++) //
            {
                SPI_FLASH_BUF[i+secoff]=pBuffer[i];
            }

            SPI_Flash_Write_NoCheck(SPI_FLASH_BUF,secpos*4096,4096); //д

        }
        else
        {
            SPI_Flash_Write_NoCheck(pBuffer,WriteAddr,secremain); //дѾ˵ģֱдʣռ
        }

        if(size==secremain)
        {
            break; //д
        }
        else
        {
            secpos++; //ַ1
            secoff=0; //ƫλΪ0

          pBuffer+=secremain;   //ָƫ
          WriteAddr+=secremain; //дַƫ
          size-=secremain;      //ֽݼ

            if(size>4096)
            {
                secremain=4096; //һд
            }
            else
            {
                secremain=size; //һд
            }
        }
    }
}

/*******************************************************************************
* Function Name  : SPI_Flash_Erase_Chip
* Description    : Erase all FLASH pages.
* Input          : None
* Return         : None
*******************************************************************************/
void SPI_Flash_Erase_Chip(void)
{
  SPI_FLASH_Write_Enable();
  SPI_Flash_Wait_Busy();
  GPIO_WriteBit(CS_GPIO, CS_PIN, 0);
  SPI_ReadWriteByte(W25X_ChipErase);  //Ƭָ
  GPIO_WriteBit(CS_GPIO, CS_PIN, 1);
  SPI_Flash_Wait_Busy();
}

/*******************************************************************************
* Function Name  : SPI_Flash_PowerDown
* Description    : Enter power down mode.
* Input          : None
* Return         : None
*******************************************************************************/
void SPI_Flash_PowerDown(void)
{
  GPIO_WriteBit(CS_GPIO, CS_PIN, 0);
  SPI_ReadWriteByte(W25X_PowerDown);  //ͽϵģʽ֮
  GPIO_WriteBit(CS_GPIO, CS_PIN, 1);
  Delay_Us(3);
}

/*******************************************************************************
* Function Name  : SPI_Flash_WAKEUP
* Description    : Power down wake up.
* Input          : None
* Return         : None
*******************************************************************************/
void SPI_Flash_WAKEUP(void)
{
  GPIO_WriteBit(CS_GPIO, CS_PIN, 0);
  SPI_ReadWriteByte(W25X_ReleasePowerDown); //Ͷϵ绽ָ
  GPIO_WriteBit(CS_GPIO, CS_PIN, 1);
  Delay_Us(3);
}

void SPI_Flash_TEST(void)
{
    const int SIZE = 100;
    u8 TEXT_Buf[100] = "hello world \n";
    u8 datap[SIZE];
    u16 Flash_Model;
    SPI_Flash_Init();
    Flash_Model = SPI_Flash_ReadID();
    printf("SPI_Flash_ReadID = %x \n", Flash_Model);
    switch(Flash_Model)
    {
    case W25Q80:
        printf("W25Q80 OK!\r\n");
        break;
    case W25Q16:
        printf("W25Q16 OK!\r\n");
        break;
    case W25Q32:
        printf("W25Q32 OK!\r\n");
        break;
    case W25Q64:
        printf("W25Q64 OK!\r\n");
        break;
    case W25Q128:
        printf("W25Q128 OK!\r\n");
        break;
    default:
        printf("Fail!\r\n");
      break;
    }

    printf("Start Erase W25Qxx....\r\n");
    SPI_Flash_Erase_Sector(0);
    printf("W25Qxx Erase Finished!\r\n");

    Delay_Ms(500);
    printf("Start Write W25Qxx....\r\n");
    SPI_Flash_Write((u8*)TEXT_Buf,0,SIZE);
    printf("W25Qxx Write Finished!\r\n");

    Delay_Ms(500);
    printf("Start Read W25Qxx....\r\n");
    SPI_Flash_Read(datap,0x0,SIZE);
    printf("READ:%s\r\n", datap );
}
